import re
from abc import ABC
from datetime import date
from typing import List
from urllib import request
from babel.dates import format_date
from bs4 import BeautifulSoup
from .event import Event


class Source(ABC):
    def __init__(self) -> None:
        self.today = date.today()

    def parse(self) -> List[Event]:
        pass  # interface method

    @staticmethod
    def _get_soup(url: str) -> BeautifulSoup:
        # Validate URL before opening it
        if url.lower().startswith("https"):
            req = request.Request(url)
        else:
            raise ValueError("Not going to get resource from unsecure URL")

        with request.urlopen(req) as body:  # skipcq: BAN-B310
            return BeautifulSoup(body, "html.parser")


class Wikipedia(Source):
    URL_PREFIX = "https://it.wikipedia.org/wiki"

    @staticmethod
    def _sanitize(text: str) -> str:
        return re.sub(r"\[\d+]", "", text)

    def _to_event(self, year: str, title: str) -> Event:
        title = self._sanitize(title)

        if "a.C." in year:
            year = int(re.sub(r"a\.C\.", "", year))
            return Event(year=year, title=title, bc=True)

        return Event(year=int(year), title=title)

    def parse(self) -> List[Event]:
        day = format_date(self.today, "d_MMMM", locale="it")
        url = f"{self.URL_PREFIX}/{day}"

        soup = self._get_soup(url)

        event_title = soup.find(id="Eventi").parent
        event_list = event_title.find_next_sibling().find_all("li", recursive=False)

        res = []

        for event in event_list:

            if subevents := event.find("ul"):
                # Multiline event
                year = event.a.text
                for subevent in subevents.find_all("li"):
                    res.append(self._to_event(year, subevent.text))
            else:
                # Simple event
                year, title = re.split("-|–", event.text, maxsplit=1)
                res.append(self._to_event(year, title))

        return res


class AccaddeOggi(Source):
    URL = "https://www.accaddeoggi.it"

    @staticmethod
    def _to_event(year: str, title: str) -> Event:
        return Event(year=int(year.strip()), title=title.strip())

    def parse(self) -> List[Event]:
        soup = self._get_soup(self.URL)

        stop_tag = soup.find(text="Sono nati oggi")

        event_list = stop_tag.find_all_previous("div", class_="entry")

        res = []
        for event in event_list:
            text = event.find("a").text
            year, title = re.split(" - | – ", text, maxsplit=1)
            res.append(self._to_event(year, title))

        res.reverse()

        return res



from dataclasses import dataclass


@dataclass
class Event:
    year: int
    title: str
    bc: bool = False

    def __str__(self) -> str:
        return f'[{self.year}{" a.C." if self.bc else ""}] ' f"{self.title}"

    def get_year(self) -> int:
        return self.year if not self.bc else -self.year



from typing import Optional

from typer import Typer

from onthisday import Today

app = Typer()


@app.command(name="all")
def all_(source: str = "wiki") -> None:
    """Print all the events of today"""
    event_source = Today(source)
    print(*event_source.all(), sep="\n")


@app.command(name="last")
def last_(
    n: int = 1,
    source: str = "wiki",
    from_year: Optional[str] = None,
    to_year: Optional[str] = None,
) -> None:
    """Print last n events of today"""
    event_source = Today(source)
    print(*event_source.last(n, from_year=from_year, to_year=to_year), sep="\n")


@app.command(name="random")
def random_(
    source: str = "wiki", from_year: Optional[str] = None, to_year: Optional[str] = None
) -> None:
    """Print random event of today"""
    event_source = Today(source)
    print(event_source.random(from_year=from_year, to_year=to_year))


@app.command(name="list")
def list_() -> None:
    """List the available sources (valid shorter names in brackets)"""
    available_sources = ["Wikipedia (wiki)", "Accadde Oggi (accadde)"]
    print("\nAvailable sources:\n")

    for source in available_sources:
        print(f"    • {source}")


# Launch as a standalone script
if __name__ == "__main__":
    app()



from setuptools import find_packages, setup

with open("README.md") as f:
    readme = f.read()

setup(
    name="onthisday",
    packages=find_packages(include=["onthisday"]),
    version="1.0.2",
    description="Retrieve events that happened on current day",
    long_description=readme,
    long_description_content_type="text/markdown",
    author="bit97",
    author_email="federico.bitondo@gmail.com",
    url="https://github.com/bit97/onthisday",
    download_url="https://github.com/bit97/onthisday/archive/refs/tags/1.0.1.tar.gz",
    keywords=[
        "cli",
        "command-line",
        "wikipedia",
        "scraping",
        "today-widget",
        "today",
        "greetings",
    ],
    license="WTFPL",
    install_requires=["Babel", "beautifulsoup4", "typer"],
    setup_requires=[],
    tests_require=[],
    test_suite="tests",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Libraries",
        "Topic :: Utilities",
        "Environment :: Console",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
    ],
)



from random import choice
from typing import List

from .event import Event
from .source import AccaddeOggi, Wikipedia


class Today:
    def __init__(self, source_name: str = "wiki") -> None:
        source_name = source_name.lower()
        if "wiki" in source_name:
            self.source = Wikipedia()
        elif "accadde" in source_name:
            self.source = AccaddeOggi()
        else:
            raise NotImplementedError(f"Unsupported source type {source_name}")

    def _retrieve(self, **kwargs) -> List[Event]:
        events = self.source.parse()

        if "from_year" in kwargs and kwargs["from_year"]:
            events = [
                event
                for event in events
                if event.get_year() >= int(kwargs["from_year"])
            ]

        if "to_year" in kwargs and kwargs["to_year"]:
            events = [
                event for event in events if event.get_year() <= int(kwargs["to_year"])
            ]

        return events

    def last(self, n: int = 1, **kwargs) -> List[Event]:
        events = self._retrieve(**kwargs)
        return events[slice(abs(n) * -1, None)]

    def random(self, **kwargs) -> Event:
        events = self._retrieve(**kwargs)

        return choice(events)

    def all(self) -> List[Event]:
        return self._retrieve()
